/*
       .
.

     Include     :
#include <SignalStatement.mqh>

     Orders-  S._Orders- (   (Ctrl + H)).
     Account-  S._Account- (   (Ctrl + H)).

    ->  ->   ' WebRequest   URL:'
    :
  https://www.mql5.com
  https://login.mql5.com
*/

input int SignalID = 0;
input string MQL5_Login = "";
input string MQL5_Password = "";
input int ShiftTime = 0; // ShiftTime in Minutes

class STRING
{
public:
  static string GetBetweenString( string &SourceString, const string BeginString, const string EndString = "" )
  {
    string Str = SourceString;
    int Pos1 = 0;
    int Pos2 = 0;

    if (BeginString != "")
      Pos1 = StringFind(SourceString, BeginString);

    if (Pos1 >= 0)
    {
      Pos1 += StringLen(BeginString);

      Pos2 = StringFind(SourceString, EndString, Pos1);

      if (Pos2 != Pos1)
        Str = StringSubstr(SourceString, Pos1, Pos2 - Pos1);
      else
        Str = "";
    }

    SourceString = StringSubstr(SourceString, Pos1 + Pos2 + StringLen(EndString));

    return(Str);
  }
};

#define GETSTATEMENT_MACROS(METHOD)                                                                       \
{                                                                                                         \
  ResetLastError();                                                                                       \
                                                                                                          \
  if (WebRequest(METHOD, URL, Cookie, NULL, 10000, Post, ArraySize(Post), Result, ResponseHeaders) == -1) \
  {                                                                                                       \
    Alert("Error in WebRequest. Error code  = " + (string)GetLastError());                                \
    Alert("Add address '" + URL + "' in Expert Advisors tab of the Options window");                      \
                                                                                                          \
    return(Res);                                                                                          \
  }                                                                                                       \
}                                                                                                         \

#define OP_BALANCE 6
#define SETDATA_MACROS STRING::GetBetweenString(Statement, "", Delimeter)

struct SIGNALORDER
{
  datetime OpenTime;
  int Type;
  double volume;
  string symbol;
  double OpenPrice;
  double SL;
  double TP;
  datetime CloseTime;
  double ClosePrice;
  double Commission;
  double Swap;
  double Profit;
  string comment;
  int Ticket;

  void SetData( string &Statement, const int TimeShift = 0, const string Delimeter = ";" )
  {
    const string sType[] = {"Buy", "Sell", "Buy Limit", "Sell Limit", "Buy Stop", "Sell Stop", "Balance"};
    const int iType[] = {OP_BUY, OP_SELL, OP_BUYLIMIT, OP_SELLLIMIT, OP_BUYSTOP, OP_SELLSTOP, OP_BALANCE};

    OpenTime = StringToTime(SETDATA_MACROS) + TimeShift * PERIOD_H1;

    string Tmp = STRING::GetBetweenString(Statement, "", Delimeter);

    for (int i = 0; i < ArraySize(iType); i++)
      if (Tmp == sType[i])
      {
        Type = iType[i];

        break;
      }

    volume = StringToDouble(SETDATA_MACROS);
    symbol = SETDATA_MACROS;
    OpenPrice = StringToDouble(SETDATA_MACROS);
    SL = StringToDouble(SETDATA_MACROS);
    TP = StringToDouble(SETDATA_MACROS);
    CloseTime = StringToTime(SETDATA_MACROS) + TimeShift * PERIOD_H1;
    ClosePrice = StringToDouble(SETDATA_MACROS);
    Commission = StringToDouble(SETDATA_MACROS);
    Swap = StringToDouble(SETDATA_MACROS);
    Profit = StringToDouble(SETDATA_MACROS);

    comment = STRING::GetBetweenString(Statement, "", "\r\n");

    if (CloseTime < OpenTime)
      CloseTime = OpenTime;

    return;
  }
};

class SIGNAL
{
protected:
  const string MQL5Login;
  const string MQL5Password;

  SIGNALORDER Orders[];
  int Amount;
  int Select;

  string Company;
  string Server;
  string Name;
  int Number;
  double Balance;
  double Equity;
  int Leverage;
  string Currency;

  static bool SelectByID( const long ID )
  {
    bool Res = FALSE;

    for (int i = SignalBaseTotal() - 1; i >= 0; i--)
      if (SignalBaseSelect(i))
      {
        Res = (ID == SignalBaseGetInteger(SIGNAL_BASE_ID));

        if (Res)
          break;
      }

    return(Res);
  }

  static string GetStatement( const int ID, const string Login, const string Password, string &sCurrency )
  {
    string Res = "";
    string ResponseHeaders;
    string Cookie = "";
    string URL = "https://login.mql5.com/en/auth_login";
    char Result[];
    char Post[];

    StringToCharArray("Login=" + Login + "&Password=" + Password + "&RememberMe=false", Post);

    GETSTATEMENT_MACROS("POST")

    Cookie = STRING::GetBetweenString(ResponseHeaders, "Set-Cookie: ", ";");
    URL = "https://www.mql5.com/en/signals/" + (string)ID;

    GETSTATEMENT_MACROS("GET")

    Res = CharArrayToString(Result);

    sCurrency = STRING::GetBetweenString(Res, "Balance:\r\n                <span>", "</span>");
    sCurrency = STRING::GetBetweenString(sCurrency, " ");
    sCurrency = STRING::GetBetweenString(sCurrency, ".");
    sCurrency = STRING::GetBetweenString(sCurrency, " ");

    URL = URL + "/export/history";

    GETSTATEMENT_MACROS("GET")

    Res = CharArrayToString(Result);

    return(Res);
  }

public:
  SIGNAL( const int ID, const string Login, const string Password, const int TimeShift = 0 ) : MQL5Login(Login), MQL5Password(Password)
  {
    Init(ID, TimeShift);
  }

  bool Init( const int ID, const int TimeShift = 0 )
  {
    if (!SelectByID(ID))
    {
      Alert("API Signals: not found " + (string)ID + "-signal.");

      return(FALSE);
    }

    string Statement = GetStatement(ID, MQL5Login, MQL5Password, Currency);

    STRING::GetBetweenString(Statement, "", "\r\n");

    Amount = 0;

    while (Statement != "")
    {
      ArrayResize(Orders, Amount + 1);

      Orders[Amount].SetData(Statement, TimeShift);
      Orders[Amount].Ticket = Amount + 1;

      Amount++;
    }

    Select = 0;

    Balance = SignalBaseGetDouble(SIGNAL_BASE_BALANCE);
    Equity = SignalBaseGetDouble(SIGNAL_BASE_EQUITY);
    Name = SignalBaseGetString(SIGNAL_BASE_NAME) + " (" +
           SignalBaseGetString(SIGNAL_BASE_AUTHOR_LOGIN) + ") URL: http://www.mql5.com/en/signals/" + (string)ID;
    Company = SignalBaseGetString(SIGNAL_BASE_BROKER);
    Server = SignalBaseGetString(SIGNAL_BASE_BROKER_SERVER);
    Leverage = (int)SignalBaseGetInteger(SIGNAL_BASE_LEVERAGE);
    Number = ID;

    return(TRUE);
  }

  int _OrdersHistoryTotal( void ) const
  {
    return(Amount);
  }

  bool _OrderSelect( int index, int select, int pool = MODE_TRADES )
  {
    bool Res;

    if (pool == MODE_TRADES )
      return(OrderSelect(index, select, pool));

    if (select == SELECT_BY_TICKET)
    {
      Res = (index <= Amount) && (index > 0);

      if (Res)
        Select = index - 1;
    }
    else
    {
      Res = (index < Amount) && (index >= 0);

      if (Res)
        Select = index;
    }

    return(Res);
  }

  string _OrderSymbol( void ) const
  {
    return(Orders[Select].symbol);
  }

  double _OrderOpenPrice( void ) const
  {
    return(Orders[Select].OpenPrice);
  }

  double _OrderClosePrice( void ) const
  {
    return(Orders[Select].ClosePrice);
  }

  double _OrderTakeProfit( void ) const
  {
    return(Orders[Select].TP);
  }

  double _OrderStopLoss( void ) const
  {
    return(Orders[Select].SL);
  }

  datetime _OrderOpenTime( void ) const
  {
    return(Orders[Select].OpenTime);
  }

  datetime _OrderCloseTime( void ) const
  {
    return(Orders[Select].CloseTime);
  }

  double _OrderLots( void ) const
  {
    return(Orders[Select].volume);
  }

  int _OrderType( void ) const
  {
    return(Orders[Select].Type);
  }

  double _OrderCommission( void ) const
  {
    return(Orders[Select].Commission);
  }

  double _OrderSwap( void ) const
  {
    return(Orders[Select].Swap);
  }

  double _OrderProfit( void ) const
  {
    return(Orders[Select].Profit);
  }

  string _OrderComment( void ) const
  {
    return(Orders[Select].comment);
  }

  int _OrderTicket( void )
  {
    return(Orders[Select].Ticket);
  }

  static int _OrderMagicNumber( void )
  {
    return(0);
  }

  static datetime _OrderExpiration( void )
  {
    return(0);
  }

  double _AccountBalance( void ) const
  {
    return(Balance);
  }

  double _AccountEquity( void ) const
  {
    return(Equity);
  }

  double _AccountProfit( void ) const
  {
    return(Equity - Balance);
  }

  int _AccountNumber( void ) const
  {
    return(Number);
  }

  int _AccountLeverage( void ) const
  {
    return(Leverage);
  }

  string _AccountName( void ) const
  {
    return(Name);
  }

  string _AccountCompany( void ) const
  {
    return(Company);
  }

  string _AccountServer( void ) const
  {
    return(Server);
  }

  string _AccountCurrency( void ) const
  {
    return(Currency);
  }
};

SIGNAL S(SignalID, MQL5_Login, MQL5_Password, ShiftTime);